"""
Vector Operators

# TODO: 没有搞明白，等看懂了再来实现
"""


def vec_avg(x):
    """Taking mean of the vector field x"""
    pass


def vec_choose(x, nth=1):
    """Choosing kth item(indexed at 0) from each vector field x"""
    pass


def vec_count(x):
    """Number of elements in vector field x"""
    pass


def vec_ir(x):
    """Information Ratio (Mean / Standard Deviation) of vector field x"""
    pass


def vec_kurtosis(x):
    """Kurtosis of vector field x"""
    pass


def vec_max(x):
    """Maximum value form vector field x"""
    pass


def vec_min(x):
    """Minimum value form vector field x"""
    pass


def vec_norm(x):
    """Sum of all absolute values of vector field x"""
    pass


def vec_percentage(x, percentage=0.5):
    """Percentile of vector field x"""
    pass


def vec_powersum(x, constant=2):
    """Sum of power of vector field x"""
    pass


def vec_range(x):
    """Difference between maximum and minimum element in vector field x"""
    pass


def vec_skewness(x):
    """Skewness of vector field x"""
    pass


def vec_stddev(x):
    """Standard Deviation of vector field x"""
    pass


def vec_sum(x):
    """Sum of vector field x"""
    pass
