"""
Special Operators
"""


def convert(x, mode="dollar2share"):
    """Convert dollars to share or share to dollar when mode = "share2dollar"
    """
    pass


def inst_pnl(x):
    """Generate pnl per instruments"""
    pass
