"""
本项目参考了WorldQuant BRAIN中Fast Expression的语法

References
----------
1. https://platform.worldquantbrain.com/learn/data-and-operators/operators
2. https://platform.worldquantbrain.com/learn/data-and-operators/detailed-operator-descriptions
"""
